@@ -3,22 +3,14 @@ package App::Netdisco::Core::Discover;
 use Dancer qw/:syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
-use App::Netdisco::Util::Device
-  qw/get_device match_devicetype is_discoverable/;
-use App::Netdisco::Util::Permission 'check_acl_only';
+use App::Netdisco::Util::Device qw/get_device is_discoverable/;
 use App::Netdisco::Util::DNS ':all';
-use App::Netdisco::JobQueue qw/jq_queued jq_insert/;
 use NetAddr::IP::Lite ':lower';
-use List::MoreUtils ();
-use Scalar::Util 'blessed';
-use Encode;
 use Try::Tiny;
-use NetAddr::MAC;
 
 use base 'Exporter';
 our @EXPORT = ();
 our @EXPORT_OK = qw/
-  set_canonical_ip
   store_device store_interfaces store_wireless
   store_vlans store_power store_modules
   store_neighbors discover_new_neighbors
@@ -38,73 +30,6 @@ subroutines.
 
 =head1 EXPORT_OK
 
-=head2 set_canonical_ip( $device, $snmp )
-
-Returns: C<$device>
-
-Given a Device database object, and a working SNMP connection, check whether
-the database object's IP is the best choice for that device. If not, update
-the IP and hostname in the device object for the canonical IP.
-
-=cut
-
-sub set_canonical_ip {
-  my ($device, $snmp) = @_;
-
-  my $old_ip = $device->ip;
-  my $new_ip = $old_ip;
-  my $revofname = ipv4_from_hostname($snmp->name);
-
-  if (setting('reverse_sysname') and $revofname) {
-    if ($snmp->snmp_connect_ip( $new_ip )) {
-      $new_ip = $revofname;
-    }
-    else {
-      debug sprintf ' [%s] device - cannot renumber to %s - SNMP connect failed',
-        $old_ip, $revofname;
-    }
-  }
-
-  if (scalar @{ setting('device_identity') }) {
-    my @idmaps = @{ setting('device_identity') };
-    my $devips = $device->device_ips->order_by('alias');
-
-    ALIAS: while (my $alias = $devips->next) {
-      next if $alias->alias eq $old_ip;
-
-      foreach my $map (@idmaps) {
-        next unless ref {} eq ref $map;
-
-        foreach my $key (sort keys %$map) {
-          # lhs matches device, rhs matches device_ip
-          if (check_acl_only($device, $key)
-                and check_acl_only($alias, $map->{$key})) {
-
-            if ($snmp->snmp_connect_ip( $alias->alias )) {
-              $new_ip = $alias->alias;
-              last ALIAS;
-            }
-            else {
-              debug sprintf ' [%s] device - cannot renumber to %s - SNMP connect failed',
-                $old_ip, $alias->alias;
-            }
-          }
-        }
-      }
-    } # ALIAS
-  }
-
-  return if $new_ip eq $old_ip;
-
-  schema('netdisco')->txn_do(sub {
-    $device->renumber($new_ip)
-      or die "cannot renumber to: $new_ip"; # rollback
-
-    debug sprintf ' [%s] device - changed IP to %s (%s)',
-      $old_ip, $device->ip, ($device->dns || '');
-  });
-}
-
 =head2 store_device( $device, $snmp )
 
 Given a Device database object, and a working SNMP connection, discover and
@@ -122,13 +47,17 @@ sub store_device {
   my $interfaces = $snmp->interfaces;
   my $ip_netmask = $snmp->ip_netmask;
 
+  # find root IP
+  _set_canonical_ip($device, $snmp);
+
+  my $hostname = hostname_from_ip($device->ip);
+  $device->dns($hostname) if $hostname;
   my $localnet = NetAddr::IP::Lite->new('127.0.0.0/8');
 
   # build device aliases suitable for DBIC
   my @aliases;
   foreach my $entry (keys %$ip_index) {
-      my $ip = NetAddr::IP::Lite->new($entry)
-        or next;
+      my $ip = NetAddr::IP::Lite->new($entry);
       my $addr = $ip->addr;
 
       next if $addr eq '0.0.0.0';
@@ -146,61 +75,79 @@ sub store_device {
           alias => $addr,
           port => $port,
           subnet => $subnet,
-          dns => undef,
+          dns => hostname_from_ip($addr),
       };
   }
 
-  debug sprintf ' resolving %d aliases with max %d outstanding requests',
-      scalar @aliases, $ENV{'PERL_ANYEVENT_MAX_OUTSTANDING_DNS'};
-  my $resolved_aliases = hostnames_resolve_async(\@aliases);
-
-  # fake one aliases entry for devices not providing ip_index
-  push @$resolved_aliases, { alias => $device->ip, dns => $device->dns }
-    if 0 == scalar @aliases;
-
   # VTP Management Domain -- assume only one.
   my $vtpdomains = $snmp->vtp_d_name;
   my $vtpdomain;
   if (defined $vtpdomains and scalar values %$vtpdomains) {
-      $device->set_column( vtp_domain => (values %$vtpdomains)[-1] );
+      $device->vtp_domain( (values %$vtpdomains)[-1] );
   }
 
-  my $hostname = hostname_from_ip($device->ip);
-  $device->set_column( dns => $hostname ) if $hostname;
-
   my @properties = qw/
-    snmp_ver
-    description uptime name
-    layers ports mac
+    snmp_ver snmp_comm
+    description uptime contact name location
+    layers ports mac serial model
     ps1_type ps2_type ps1_status ps2_status
     fan slots
     vendor os os_ver
   /;
 
   foreach my $property (@properties) {
-      $device->set_column( $property => $snmp->$property );
+      $device->$property( $snmp->$property );
   }
 
-  $device->set_column( model  => Encode::decode('UTF-8', $snmp->model)  );
-  $device->set_column( serial => Encode::decode('UTF-8', $snmp->serial) );
-  $device->set_column( contact => Encode::decode('UTF-8', $snmp->contact) );
-  $device->set_column( location => Encode::decode('UTF-8', $snmp->location) );
-
-
-  $device->set_column( snmp_class => $snmp->class );
-  $device->set_column( last_discover => \'now()' );
+  $device->snmp_class( $snmp->device_type );
+  $device->last_discover(\'now()');
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->device_ips->delete;
-    debug sprintf ' [%s] device - removed %d aliases',
+    debug sprintf ' [%s] device - removed %s aliases',
       $device->ip, $gone;
     $device->update_or_insert(undef, {for => 'update'});
-    $device->device_ips->populate($resolved_aliases);
+    $device->device_ips->populate(\@aliases);
     debug sprintf ' [%s] device - added %d new aliases',
       $device->ip, scalar @aliases;
   });
 }
 
+sub _set_canonical_ip {
+  my ($device, $snmp) = @_;
+
+  my $oldip = $device->ip;
+  my $newip = $snmp->root_ip;
+
+  if (defined $newip) {
+      if ($oldip ne $newip) {
+          debug sprintf ' [%s] device - changing root IP to alt IP %s',
+            $oldip, $newip;
+
+          # remove old device and aliases
+          schema('netdisco')->txn_do(sub {
+            my $gone = $device->device_ips->delete;
+            debug sprintf ' [%s] device - removed %s aliases',
+              $oldip, $gone;
+            $device->delete;
+            debug sprintf ' [%s] device - deleted self', $oldip;
+          });
+
+          $device->ip($newip);
+      }
+
+      # either root_ip is changed or unchanged, but it exists
+      return;
+  }
+
+  my $revname = ipv4_from_hostname($snmp->name);
+  if (setting('reverse_sysname') and $revname) {
+      debug sprintf ' [%s] device - changing root IP to revname %s',
+        $oldip, $revname;
+      $device->ip($revname);
+  }
+}
+
 =head2 store_interfaces( $device, $snmp )
 
 Given a Device database object, and a working SNMP connection, discover and
@@ -228,39 +175,26 @@ sub store_interfaces {
   my $i_duplex_admin = $snmp->i_duplex_admin;
   my $i_stp_state    = $snmp->i_stp_state;
   my $i_vlan         = $snmp->i_vlan;
+  my $i_pvid         = $snmp->i_pvid;
   my $i_lastchange   = $snmp->i_lastchange;
-  my $agg_ports      = $snmp->agg_ports;
 
   # clear the cached uptime and get a new one
   my $dev_uptime = $snmp->load_uptime;
-  if (!defined $dev_uptime) {
-      error sprintf ' [%s] interfaces - Error! Failed to get uptime from device!',
+
+  if (scalar grep {$_ > $dev_uptime} values %$i_lastchange) {
+      info sprintf ' [%s] interfaces - device uptime has wrapped - correcting',
         $device->ip;
-      return;
-  }
-
-  # used to track how many times the device uptime wrapped
-  my $dev_uptime_wrapped = 0;
-
-  # use SNMP-FRAMEWORK-MIB::snmpEngineTime if available to
-  # fix device uptime if wrapped
-  if (defined $snmp->snmpEngineTime) {
-      $dev_uptime_wrapped = int( $snmp->snmpEngineTime * 100 / 2**32 );
-      if ($dev_uptime_wrapped > 0) {
-          info sprintf ' [%s] interface - device uptime wrapped %d times - correcting',
-            $device->ip, $dev_uptime_wrapped;
-          $device->uptime( $dev_uptime + $dev_uptime_wrapped * 2**32 );
-      }
+      $device->uptime( $dev_uptime + 2**32 );
   }
 
   # build device interfaces suitable for DBIC
-  my %interfaces;
+  my @interfaces;
   foreach my $entry (keys %$interfaces) {
       my $port = $interfaces->{$entry};
 
-      if (not $port) {
+      if (not length $port) {
           debug sprintf ' [%s] interfaces - ignoring %s (no port mapping)',
-            $device->ip, $entry;
+            $device->ip, $port;
           next;
       }
 
@@ -277,14 +211,7 @@ sub store_interfaces {
           next;
       }
 
-      my $lc = $i_lastchange->{$entry} || 0;
-      if (not $dev_uptime_wrapped and $lc > $dev_uptime) {
-          info sprintf ' [%s] interfaces - device uptime wrapped (%s) - correcting',
-            $device->ip, $port;
-          $device->uptime( $dev_uptime + 2**32 );
-          $dev_uptime_wrapped = 1;
-      }
-
+      my $lc = $i_lastchange->{$entry};
       if ($device->is_column_changed('uptime') and $lc) {
           if ($lc < $dev_uptime) {
               # ambiguous: lastchange could be sysUptime before or after wrap
@@ -298,12 +225,12 @@ sub store_interfaces {
                   debug sprintf
                     ' [%s] interfaces - correcting LastChange for %s, assuming sysUptime wrap',
                     $device->ip, $port;
-                  $lc += $dev_uptime_wrapped * 2**32;
+                  $lc += 2**32;
               }
           }
       }
 
-      $interfaces{$port} = {
+      push @interfaces, {
           port         => $port,
           descr        => $i_descr->{$entry},
           up           => $i_up->{$entry},
@@ -311,37 +238,25 @@ sub store_interfaces {
           mac          => $i_mac->{$entry},
           speed        => $i_speed->{$entry},
           mtu          => $i_mtu->{$entry},
-          name         => Encode::decode('UTF-8', $i_name->{$entry}),
+          name         => $i_name->{$entry},
           duplex       => $i_duplex->{$entry},
           duplex_admin => $i_duplex_admin->{$entry},
           stp          => $i_stp_state->{$entry},
           type         => $i_type->{$entry},
           vlan         => $i_vlan->{$entry},
-          pvid         => $i_vlan->{$entry},
-          is_master    => 'false',
-          slave_of     => undef,
+          pvid         => $i_pvid->{$entry},
           lastchange   => $lc,
       };
   }
 
-  # must do this after building %interfaces so that we can set is_master
-  foreach my $sidx (keys %$agg_ports) {
-      my $slave  = $interfaces->{$sidx} or next;
-      my $master = $interfaces->{ $agg_ports->{$sidx} } or next;
-      next unless exists $interfaces{$slave} and exists $interfaces{$master};
-
-      $interfaces{$slave}->{slave_of} = $master;
-      $interfaces{$master}->{is_master} = 'true';
-  }
-
-  schema('netdisco')->resultset('DevicePort')->txn_do_locked(sub {
-    my $gone = $device->ports->delete({keep_nodes => 1});
-    debug sprintf ' [%s] interfaces - removed %d interfaces',
+  schema('netdisco')->txn_do(sub {
+    my $gone = $device->ports->delete;
+    debug sprintf ' [%s] interfaces - removed %s interfaces',
       $device->ip, $gone;
     $device->update_or_insert(undef, {for => 'update'});
-    $device->ports->populate([values %interfaces]);
+    $device->ports->populate(\@interfaces);
     debug sprintf ' [%s] interfaces - added %d new interfaces',
-      $device->ip, scalar values %interfaces;
+      $device->ip, scalar @interfaces;
   });
 }
 
@@ -373,9 +288,9 @@ sub store_wireless {
       (my $iid = $entry) =~ s/\.\d+$//;
       my $port = $interfaces->{$iid};
 
-      if (not $port) {
+      if (not length $port) {
           debug sprintf ' [%s] wireless - ignoring %s (no port mapping)',
-            $device->ip, $iid;
+            $device->ip, $port;
           next;
       }
 
@@ -389,7 +304,7 @@ sub store_wireless {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->ssids->delete;
-    debug sprintf ' [%s] wireless - removed %d SSIDs',
+    debug sprintf ' [%s] wireless - removed %s SSIDs',
       $device->ip, $gone;
     $device->ssids->populate(\@ssids);
     debug sprintf ' [%s] wireless - added %d new SSIDs',
@@ -401,9 +316,9 @@ sub store_wireless {
   foreach my $entry (keys %$channel) {
       my $port = $interfaces->{$entry};
 
-      if (not $port) {
+      if (not length $port) {
           debug sprintf ' [%s] wireless - ignoring %s (no port mapping)',
-            $device->ip, $entry;
+            $device->ip, $port;
           next;
       }
 
@@ -416,7 +331,7 @@ sub store_wireless {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->wireless_ports->delete;
-    debug sprintf ' [%s] wireless - removed %d wireless channels',
+    debug sprintf ' [%s] wireless - removed %s wireless channels',
       $device->ip, $gone;
     $device->wireless_ports->populate(\@channels);
     debug sprintf ' [%s] wireless - added %d new wireless channels',
@@ -445,7 +360,6 @@ sub store_vlans {
   my @devicevlans;
   foreach my $entry (keys %$v_name) {
       my $vlan = $v_index->{$entry};
-      next unless defined $vlan and $vlan;
       ++$v_seen{$vlan};
 
       push @devicevlans, {
@@ -461,18 +375,14 @@ sub store_vlans {
   my $interfaces        = $snmp->interfaces;
 
   # build device port vlans suitable for DBIC
-  my @portvlans = ();
+  my @portvlans;
   foreach my $entry (keys %$i_vlan_membership) {
-      my %port_vseen = ();
       my $port = $interfaces->{$entry};
       next unless defined $port;
 
       my $type = $i_vlan_type->{$entry};
 
       foreach my $vlan (@{ $i_vlan_membership->{$entry} }) {
-          next unless defined $vlan and $vlan;
-          next if ++$port_vseen{$vlan} > 1;
-
           my $native = ((defined $i_vlan->{$entry}) and ($vlan eq $i_vlan->{$entry})) ? "t" : "f";
           push @portvlans, {
               port => $port,
@@ -496,7 +406,7 @@ sub store_vlans {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->vlans->delete;
-    debug sprintf ' [%s] vlans - removed %d device VLANs',
+    debug sprintf ' [%s] vlans - removed %s device VLANs',
       $device->ip, $gone;
     $device->vlans->populate(\@devicevlans);
     debug sprintf ' [%s] vlans - added %d new device VLANs',
@@ -505,7 +415,7 @@ sub store_vlans {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->port_vlans->delete;
-    debug sprintf ' [%s] vlans - removed %d port VLANs',
+    debug sprintf ' [%s] vlans - removed %s port VLANs',
       $device->ip, $gone;
     $device->port_vlans->populate(\@portvlans);
     debug sprintf ' [%s] vlans - added %d new port VLANs',
@@ -572,7 +482,7 @@ sub store_power {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->power_modules->delete;
-    debug sprintf ' [%s] power - removed %d power modules',
+    debug sprintf ' [%s] power - removed %s power modules',
       $device->ip, $gone;
     $device->power_modules->populate(\@devicepower);
     debug sprintf ' [%s] power - added %d new power modules',
@@ -581,7 +491,7 @@ sub store_power {
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->powered_ports->delete;
-    debug sprintf ' [%s] power - removed %d PoE capable ports',
+    debug sprintf ' [%s] power - removed %s PoE capable ports',
       $device->ip, $gone;
     $device->powered_ports->populate(\@portpower);
     debug sprintf ' [%s] power - added %d new PoE capable ports',
@@ -602,35 +512,10 @@ not yet stored to the database.
 sub store_modules {
   my ($device, $snmp) = @_;
 
-  my $e_index = $snmp->e_index;
+  my $e_index  = $snmp->e_index;
 
   if (!defined $e_index) {
-      schema('netdisco')->txn_do(sub {
-        my $gone = $device->modules->delete;
-        debug sprintf ' [%s] modules - removed %d chassis modules',
-          $device->ip, $gone;
-
-        $device->modules->update_or_create({
-          ip => $device->ip,
-          index => 1,
-          parent => 0,
-          name => 'chassis',
-          class => 'chassis',
-          pos => -1,
-          # too verbose and link doesn't work anyway
-          # description => $device->description,
-          sw_ver => $device->os_ver,
-          serial => $device->serial,
-          model => $device->model,
-          fru => \'false',
-          last_discover => \'now()',
-        });
-      });
-
-      debug
-        sprintf ' [%s] modules - 0 chassis components (added one pseudo for chassis)',
-        $device->ip;
-
+      debug sprintf ' [%s] modules - 0 chassis components', $device->ip;
       return;
   }
 
@@ -649,28 +534,28 @@ sub store_modules {
 
   # build device modules list for DBIC
   my @modules;
-  foreach my $entry (keys %$e_index) {
+  foreach my $entry (keys %$e_class) {
       push @modules, {
           index  => $e_index->{$entry},
           type   => $e_type->{$entry},
           parent => $e_parent->{$entry},
-          name   => Encode::decode('UTF-8', $e_name->{$entry}),
+          name   => $e_name->{$entry},
           class  => $e_class->{$entry},
           pos    => $e_pos->{$entry},
-          hw_ver => Encode::decode('UTF-8', $e_hwver->{$entry}),
-          fw_ver => Encode::decode('UTF-8', $e_fwver->{$entry}),
-          sw_ver => Encode::decode('UTF-8', $e_swver->{$entry}),
-          model  => Encode::decode('UTF-8', $e_model->{$entry}),
-          serial => Encode::decode('UTF-8', $e_serial->{$entry}),
+          hw_ver => $e_hwver->{$entry},
+          fw_ver => $e_fwver->{$entry},
+          sw_ver => $e_swver->{$entry},
+          model  => $e_model->{$entry},
+          serial => $e_serial->{$entry},
           fru    => $e_fru->{$entry},
-          description => Encode::decode('UTF-8', $e_descr->{$entry}),
+          description => $e_descr->{$entry},
           last_discover => \'now()',
       };
   }
 
   schema('netdisco')->txn_do(sub {
     my $gone = $device->modules->delete;
-    debug sprintf ' [%s] modules - removed %d chassis modules',
+    debug sprintf ' [%s] modules - removed %s chassis modules',
       $device->ip, $gone;
     $device->modules->populate(\@modules);
     debug sprintf ' [%s] modules - added %d new chassis modules',
@@ -699,7 +584,7 @@ sub store_neighbors {
   my ($device, $snmp) = @_;
   my @to_discover = ();
 
-  # first allow any manually configured topology to be set
+  # first allow any manually configred topology to be set
   _set_manual_topology($device, $snmp);
 
   my $c_ip = $snmp->c_ip;
@@ -713,57 +598,22 @@ sub store_neighbors {
   my $c_port     = $snmp->c_port;
   my $c_id       = $snmp->c_id;
   my $c_platform = $snmp->c_platform;
-  my $c_cap      = $snmp->c_cap;
 
-  foreach my $entry (sort (List::MoreUtils::uniq( (keys %$c_ip), (keys %$c_cap) ))) {
-      if (!defined $c_if->{$entry} or !defined $interfaces->{ $c_if->{$entry} }) {
+  foreach my $entry (keys %$c_ip) {
+      my $port = $interfaces->{ $c_if->{$entry} };
+      if (!defined $port) {
           debug sprintf ' [%s] neigh - port for IID:%s not resolved, skipping',
             $device->ip, $entry;
           next;
       }
 
-      my $port = $interfaces->{ $c_if->{$entry} };
-      my $portrow = schema('netdisco')->resultset('DevicePort')
-          ->single({ip => $device->ip, port => $port});
-
-      if (!defined $portrow) {
-          info sprintf ' [%s] neigh - local port %s not in database!',
-            $device->ip, $port;
-          next;
-      }
-
       my $remote_ip   = $c_ip->{$entry};
       my $remote_ipad = NetAddr::IP::Lite->new($remote_ip);
       my $remote_port = undef;
-      my $remote_type = Encode::decode('UTF-8', $c_platform->{$entry} || '');
-      my $remote_id   = Encode::decode('UTF-8', $c_id->{$entry});
-      my $remote_cap  = $c_cap->{$entry} || [];
+      my $remote_type = $c_platform->{$entry};
+      my $remote_id   = $c_id->{$entry};
 
-      # IP Phone and WAP detection type fixup
-      if (scalar @$remote_cap or $remote_type) {
-          my $phone_flag = grep {match_devicetype($_, 'phone_capabilities')}
-                                @$remote_cap;
-          my $ap_flag    = grep {match_devicetype($_, 'wap_capabilities')}
-                                @$remote_cap;
-
-          if ($phone_flag or match_devicetype($remote_type, 'phone_platforms')) {
-              $remote_type = 'IP Phone: '. $remote_type
-                if $remote_type !~ /ip.phone/i;
-          }
-          elsif ($ap_flag or match_devicetype($remote_type, 'wap_platforms')) {
-              $remote_type = 'AP: '. $remote_type;
-          }
-
-          $portrow->update({remote_type => $remote_type});
-      }
-
-      if ($portrow->manual_topo) {
-          info sprintf ' [%s] neigh - %s has manually defined topology',
-            $device->ip, $port;
-          next;
-      }
-
-      next unless $remote_ip;
+      next unless length $remote_ip;
 
       # a bunch of heuristics to search known devices if we don't have a
       # useable remote IP...
@@ -778,27 +628,6 @@ sub store_neighbors {
                 ' [%s] neigh - bad address %s on port %s, searching for %s instead',
                 $device->ip, $remote_ip, $port, $remote_id;
 
-              if (!defined $neigh) {
-                  my $mac = NetAddr::MAC->new(mac => $remote_id);
-                  if ($mac and not $mac->errstr) {
-                      $neigh = $devices->single({mac => $mac->as_ieee});
-                  }
-              }
-
-              # some HP switches send 127.0.0.1 as remote_ip if no ip address
-              # on default vlan for HP switches remote_ip looks like
-              # "myswitchname(012345-012345)"
-              if (!defined $neigh) {
-                  (my $tmpid = $remote_id) =~ s/.*\(([0-9a-f]{6})-([0-9a-f]{6})\).*/$1$2/;
-                  my $mac = NetAddr::MAC->new(mac => $tmpid);
-                  if ($mac and not $mac->errstr) {
-                      info sprintf
-                        '[%s] neigh - found neighbor %s by MAC %s',
-                        $device->ip, $remote_id, $mac->as_ieee;
-                      $neigh = $devices->single({mac => $mac->as_ieee});
-                  }
-              }
-
               if (!defined $neigh) {
                   (my $shortid = $remote_id) =~ s/\..*//;
                   $neigh = $devices->single({name => { -ilike => "${shortid}%" }});
@@ -822,6 +651,15 @@ sub store_neighbors {
           }
       }
 
+      # IP Phone detection type fixup
+      if (defined $remote_type and $remote_type =~ m/(mitel.5\d{3})/i) {
+          $remote_type = 'IP Phone - '. $remote_type
+            if $remote_type !~ /ip phone/i;
+      }
+      else {
+          $remote_type = '';
+      }
+
       # hack for devices seeing multiple neighbors on the port
       if (ref [] eq ref $remote_ip) {
           debug sprintf
@@ -832,7 +670,7 @@ sub store_neighbors {
               foreach my $n (@$remote_ip) {
                   debug sprintf
                     ' [%s] neigh - adding neighbor %s, type [%s], on %s to discovery queue',
-                    $device->ip, $n, ($remote_type || ''), $port;
+                    $device->ip, $n, $remote_type, $port;
                   push @to_discover, [$n, $remote_type];
               }
           }
@@ -846,25 +684,12 @@ sub store_neighbors {
           if (wantarray) {
               debug sprintf
                 ' [%s] neigh - adding neighbor %s, type [%s], on %s to discovery queue',
-                $device->ip, $remote_ip, ($remote_type || ''), $port;
+                $device->ip, $remote_ip, $remote_type, $port;
               push @to_discover, [$remote_ip, $remote_type];
           }
 
-          # further device type discovery using MAC OUI
-          # only works once device is fully discovered (so we have a MAC addr)
-          my $neigh = get_device($remote_ip);
-          if (blessed $neigh and $neigh->in_storage and $neigh->mac) {
-              if (match_devicetype($neigh->mac, 'phone_ouis')) {
-                  $remote_type = 'IP Phone: '. $remote_type
-                    if $remote_type !~ /ip.phone/i;
-              }
-              elsif (match_devicetype($neigh->mac, 'wap_ouis')) {
-                  $remote_type = 'AP: '. $remote_type
-                    if $remote_type !~ /^AP: /;
-              }
-          }
-
           $remote_port = $c_port->{$entry};
+
           if (defined $remote_port) {
               # clean weird characters
               $remote_port =~ s/[^\d\/\.,()\w:-]+//gi;
@@ -875,6 +700,22 @@ sub store_neighbors {
           }
       }
 
+      # if all the data looks sane, update the port row with neighbor info
+      my $portrow = schema('netdisco')->resultset('DevicePort')
+          ->single({ip => $device->ip, port => $port});
+
+      if (!defined $portrow) {
+          info sprintf ' [%s] neigh - local port %s not in database!',
+            $device->ip, $port;
+          next;
+      }
+
+      if ($portrow->manual_topo) {
+          info sprintf ' [%s] neigh - %s has manually defined topology',
+            $device->ip, $port;
+          next;
+      }
+
       $portrow->update({
           remote_ip   => $remote_ip,
           remote_port => $remote_port,
@@ -883,35 +724,6 @@ sub store_neighbors {
           is_uplink   => \"true",
           manual_topo => \"false",
       });
-
-      # update master of our aggregate to be a neighbor of
-      # the master on our peer device (a lot of iffs to get there...).
-      # & cannot use ->neighbor prefetch because this is the port insert!
-      if (defined $portrow->slave_of) {
-
-          my $peer_device = get_device($remote_ip);
-          my $master = schema('netdisco')->resultset('DevicePort')->single({
-            ip => $device->ip,
-            port => $portrow->slave_of
-          });
-
-          if ($peer_device and $peer_device->in_storage and $master
-              and not ($portrow->is_master or defined $master->slave_of)) {
-
-              my $peer_port = schema('netdisco')->resultset('DevicePort')->single({
-                ip   => $peer_device->ip,
-                port => $portrow->remote_port,
-              });
-
-              $master->update({
-                  remote_ip => ($peer_device->ip || $remote_ip),
-                  remote_port => ($peer_port ? $peer_port->slave_of : undef ),
-                  is_uplink => \"true",
-                  is_master => \"true",
-                  manual_topo => \"false",
-              });
-          }
-      }
   }
 
   return @to_discover;
@@ -924,11 +736,9 @@ sub _set_manual_topology {
 
   schema('netdisco')->txn_do(sub {
     # clear manual topology flags
-    schema('netdisco')->resultset('DevicePort')
-      ->search({ip => $device->ip})->update({manual_topo => \'false'});
+    schema('netdisco')->resultset('DevicePort')->update({manual_topo => \'false'});
 
-    my $topo_links = schema('netdisco')->resultset('Topology')
-      ->search({-or => [dev1 => $device->ip, dev2 => $device->ip]});
+    my $topo_links = schema('netdisco')->resultset('Topology');
     debug sprintf ' [%s] neigh - setting manual topology links', $device->ip;
 
     while (my $link = $topo_links->next) {
@@ -943,7 +753,7 @@ sub _set_manual_topology {
               return unless ($left->in_storage and $right->in_storage);
 
               $left->ports
-                ->single({port => $link->port1})
+                ->single({port => $link->port1}, {for => 'update'})
                 ->update({
                   remote_ip => $right->ip,
                   remote_port => $link->port2,
@@ -954,7 +764,7 @@ sub _set_manual_topology {
                 });
 
               $right->ports
-                ->single({port => $link->port2})
+                ->single({port => $link->port2}, {for => 'update'})
                 ->update({
                   remote_ip => $left->ip,
                   remote_port => $link->port1,
@@ -999,15 +809,18 @@ sub discover_new_neighbors {
       if (not is_discoverable($device, $remote_type)) {
           debug sprintf
             ' queue - %s, type [%s] excluded by discover_* config',
-            $ip, ($remote_type || '');
+            $ip, $remote_type;
           next;
       }
 
-      jq_insert({
-        device => $ip,
-        action => 'discover',
-        subaction => 'with-nodes',
-      });
+      # could fail if queued job already exists
+      try {
+          schema('netdisco')->resultset('Admin')->create({
+              device => $ip,
+              action => 'discover',
+              status => 'queued',
+          });
+      };
   }
 }
 
