@@ -5,19 +5,21 @@ our @EXPORT = ();
 our @EXPORT_OK = qw/
   sort_port
 /;
-our %EXPORT_TAGS = (
-  all => [qw/
-    sort_port
-  /],
-);
+our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-=head1 App::Netdisco::Util::Web
+=head1 NAME
+
+App::Netdisco::Util::Web
+
+=head1 DESCRIPTION
 
 A set of helper subroutines to support parts of the Netdisco application.
 
 There are no default exports, however the C<:all> tag will export all
 subroutines.
 
+=head1 EXPORT_OK
+
 =head2 sort_port( $a, $b )
 
 Sort port names of various types used by device vendors. Interface is as
@@ -29,14 +31,14 @@ sub sort_port {
     my ($aval, $bval) = @_;
 
     # hack for foundry "10GigabitEthernet" -> cisco-like "TenGigabitEthernet"
-    $aval = "Ten$1" if $aval =~ qr/^10(GigabitEthernet.+)$/;
-    $bval = "Ten$1" if $bval =~ qr/^10(GigabitEthernet.+)$/;
+    $aval = $1 if $aval =~ qr/^10(GigabitEthernet.+)$/;
+    $bval = $1 if $bval =~ qr/^10(GigabitEthernet.+)$/;
 
     my $numbers        = qr{^(\d+)$};
     my $numeric        = qr{^([\d\.]+)$};
     my $dotted_numeric = qr{^(\d+)\.(\d+)$};
     my $letter_number  = qr{^([a-zA-Z]+)(\d+)$};
-    my $wordcharword   = qr{^([^:\/.]+)[\ :\/\.]+([^:\/.]+)(\d+)?$}; #port-channel45
+    my $wordcharword   = qr{^([^:\/.]+)[-\ :\/\.]+([^:\/.0-9]+)(\d+)?$}; #port-channel45
     my $netgear        = qr{^Slot: (\d+) Port: (\d+) }; # "Slot: 0 Port: 15 Gigabit - Level"
     my $ciscofast      = qr{^
                             # Word Number slash (Gigabit0/)
@@ -58,7 +60,7 @@ sub sort_port {
     } elsif ($aval =~ $numbers) {
         @a = ($1);
     } elsif ($aval =~ $ciscofast) {
-        @a = ($2,$1);
+        @a = ($1,$2);
         push @a, split(/[:\/]/,$3), $4;
     } elsif ($aval =~ $wordcharword) {
         @a = ($1,$2,$3);
@@ -75,7 +77,7 @@ sub sort_port {
     } elsif ($bval =~ $numbers) {
         @b = ($1);
     } elsif ($bval =~ $ciscofast) {
-        @b = ($2,$1);
+        @b = ($1,$2);
         push @b, split(/[:\/]/,$3),$4;
     } elsif ($bval =~ $wordcharword) {
         @b = ($1,$2,$3);
