@@ -3,10 +3,47 @@ package App::Netdisco::Web::AdminTask;
 use Dancer ':syntax';
 use Dancer::Plugin::Ajax;
 use Dancer::Plugin::DBIC;
-use Dancer::Plugin::Auth::Extensible;
 
 use Try::Tiny;
 
+sub add_job {
+    my ($jobtype, $device) = @_;
+
+    if ($device) {
+        $device = NetAddr::IP::Lite->new($device);
+        return send_error('Bad device', 400)
+          if ! $device or $device->addr eq '0.0.0.0';
+    }
+
+    try {
+    # jobs might already be in the queue, so this could die
+        schema('netdisco')->resultset('Admin')->create({
+          ($device ? (device => $device->addr) : ()),
+          action => $jobtype,
+          status => 'queued',
+          username => session('user'),
+          userip => request->remote_address,
+        });
+
+        if (param('extra') and param('extra') eq 'with-walk') {
+            schema('netdisco')->resultset('Admin')->create({
+              action => 'macwalk',
+              subaction => 'after-discoverall',
+              status => 'queued',
+              username => session('user'),
+              userip => request->remote_address,
+            });
+            schema('netdisco')->resultset('Admin')->create({
+              action => 'arpwalk',
+              subaction => 'after-discoverall',
+              status => 'queued',
+              username => session('user'),
+              userip => request->remote_address,
+            });
+        }
+    };
+}
+
 # we have a separate list for jobs needing a device to avoid queueing
 # such a job when there's no device param (it could still be duff, tho).
 my %jobs = map { $_ => 1} qw/
@@ -18,74 +55,36 @@ my %jobs_all = map {$_ => 1} qw/
     discoverall
     macwalk
     arpwalk
-    nbtwalk
 /;
 
-sub add_job {
-    my ($jobtype, $device, $subaction) = @_;
-
-    if ($device) {
-        $device = NetAddr::IP::Lite->new($device);
-        return send_error('Bad device', 400)
-          if ! $device or $device->addr eq '0.0.0.0';
-    }
-
-    # job might already be in the queue, so this could die
-    try {
-        schema('netdisco')->resultset('Admin')->create({
-          ($device ? (device => $device->addr) : ()),
-          action => $jobtype,
-          ($subaction ? (subaction => $subaction) : ()),
-          status => 'queued',
-          (exists $jobs{$jobtype} ? (username => session('logged_in_user')) : ()),
-          userip => request->remote_address,
-        });
-    };
-}
-
 foreach my $jobtype (keys %jobs_all, keys %jobs) {
-    ajax "/ajax/control/admin/$jobtype" => require_role admin => sub {
+    ajax "/ajax/control/admin/$jobtype" => sub {
+        send_error('Forbidden', 403)
+          unless var('user')->admin;
         send_error('Missing device', 400)
           if exists $jobs{$jobtype} and not param('device');
 
-        add_job($jobtype, param('device'), param('extra'));
+        add_job($jobtype, param('device'));
     };
 
-    post "/admin/$jobtype" => require_role admin => sub {
+    post "/admin/$jobtype" => sub {
+        send_error('Forbidden', 403)
+          unless var('user')->admin;
         send_error('Missing device', 400)
           if exists $jobs{$jobtype} and not param('device');
 
-        add_job($jobtype, param('device'), param('extra'));
-        redirect uri_for('/admin/jobqueue')->path;
+        add_job($jobtype, param('device'));
+        redirect uri_for('/admin/jobqueue')->path_query;
     };
 }
 
-ajax '/ajax/control/admin/delete' => require_role admin => sub {
-    send_error('Missing device', 400) unless param('device');
-
-    my $device = NetAddr::IP::Lite->new(param('device'));
-    send_error('Bad device', 400)
-      if ! $device or $device->addr eq '0.0.0.0';
-
-    schema('netdisco')->txn_do(sub {
-      schema('netdisco')->resultset('UserLog')->create({
-        username => session('logged_in_user'),
-        userip => request->remote_address,
-        event => "Delete device ". $device->addr,
-        details => param('log'),
-      });
-
-      my $device = schema('netdisco')->resultset('Device')
-        ->search({ip => param('device')});
-
-      # will delete everything related too...
-      $device->delete({archive_nodes => param('archive')});
-    });
-};
-
-get '/admin/*' => require_role admin => sub {
+get '/admin/*' => sub {
     my ($tag) = splat;
 
+    if (! eval { var('user')->admin }) {
+        return redirect uri_for('/')->path_query;
+    }
+
     # trick the ajax into working as if this were a tabbed page
     params->{tab} = $tag;
 
